require File.expand_path(File.dirname(__FILE__) + '/helper')
require 'haml'

class TennpipesPristine < Tennpipes::Application; end
class TennpipesTestApp  < Tennpipes::Application; end
class TennpipesTestApp2 < Tennpipes::Application; end

describe "Application" do
  before { Tennpipes.clear! }

  describe 'for application functionality' do

    it 'should check default options' do
      assert File.identical?(__FILE__, TennpipesPristine.app_file)
      assert_equal :tennpipes_pristine, TennpipesPristine.app_name
      assert_equal :test, TennpipesPristine.environment
      assert_equal Tennpipes.root('views'), TennpipesPristine.views
      assert  TennpipesPristine.raise_errors
      assert !TennpipesPristine.logging
      assert !TennpipesPristine.sessions
      assert !TennpipesPristine.dump_errors
      assert !TennpipesPristine.show_exceptions
      assert  TennpipesPristine.raise_errors
      assert !Tennpipes.configure_apps
    end

    it 'should check tennpipes specific options' do
      assert !TennpipesPristine.instance_variable_get(:@_configured)
      TennpipesPristine.send(:setup_application!)
      assert_equal :tennpipes_pristine, TennpipesPristine.app_name
      assert_equal 'StandardFormBuilder', TennpipesPristine.default_builder
      assert  TennpipesPristine.instance_variable_get(:@_configured)
      assert !TennpipesPristine.reload?
    end

    it 'should set global project settings' do
      Tennpipes.configure_apps { enable :sessions; set :foo, "bar" }
      TennpipesTestApp.send(:default_configuration!)
      TennpipesTestApp2.send(:default_configuration!)
      assert TennpipesTestApp.sessions, "should have sessions enabled"
      assert_equal "bar", TennpipesTestApp.settings.foo, "should have foo assigned"
      assert_equal TennpipesTestApp.session_secret, TennpipesTestApp2.session_secret
    end

    it 'should be able to configure_apps multiple times' do
      Tennpipes.configure_apps { set :foo1, "bar" }
      Tennpipes.configure_apps { set :foo1, "bam" }
      Tennpipes.configure_apps { set :foo2, "baz" }
      TennpipesTestApp.send(:default_configuration!)
      assert_equal "bam", TennpipesTestApp.settings.foo1, "should have foo1 assigned to bam"
      assert_equal "baz", TennpipesTestApp.settings.foo2, "should have foo2 assigned to baz"
    end

    it 'should have shared sessions accessible in project' do
      Tennpipes.configure_apps { enable :sessions; set :session_secret, 'secret' }
      Tennpipes.mount("TennpipesTestApp").to("/write")
      Tennpipes.mount("TennpipesTestApp2").to("/read")
      TennpipesTestApp.send :default_configuration!
      TennpipesTestApp.get('/') { session[:foo] = "shared" }
      TennpipesTestApp2.send(:default_configuration!)
      TennpipesTestApp2.get('/') { session[:foo] }
      @app = Tennpipes.application
      get '/write'
      get '/read'
      assert_equal 'shared', body
    end

    it 'should be able to execute the register keyword inside the configure_apps block' do
      Asdf = Module.new
      Tennpipes.configure_apps { register Asdf }
      class GodFather < Tennpipes::Application; end
      assert_includes GodFather.extensions, Asdf
    end

    it 'should able to set custome session management' do
      class TennpipesTestApp3 < Tennpipes::Application
        set :sessions, :use => Rack::Session::Pool
      end
      Tennpipes.mount("TennpipesTestApp3").to("/")
      TennpipesTestApp3.get('/write') { session[:foo] = "pool" }
      TennpipesTestApp3.get('/read') { session[:foo] }
      @app = Tennpipes.application
      get '/write'
      get '/read'
      assert_equal 'pool', body
    end

    it 'should have different session values in different session management' do
      class TennpipesTestApp4 < Tennpipes::Application
        enable :sessions
      end
      class TennpipesTestApp5 < Tennpipes::Application
        set :sessions, :use => Rack::Session::Pool
      end
      Tennpipes.mount("TennpipesTestApp4").to("/write")
      Tennpipes.mount("TennpipesTestApp5").to("/read")
      TennpipesTestApp4.get('/') { session[:foo] = "cookie" }
      TennpipesTestApp5.get('/') { session[:foo] }
      @app = Tennpipes.application
      get '/write'
      get '/read'
      assert_equal '', body
    end

    # compare to: test_routing: allow global provides
    it 'should set content_type to nil if none can be determined' do
      mock_app do
        provides :xml

        get("/foo"){ "Foo in #{content_type.inspect}" }
        get("/bar"){ "Foo in #{content_type.inspect}" }
      end

      get '/foo', {}, { 'HTTP_ACCEPT' => 'application/xml' }
      assert_equal 'Foo in :xml', body
      get '/foo'
      assert_equal 'Foo in :xml', body

      get '/bar', {}, { 'HTTP_ACCEPT' => 'application/xml' }
      assert_equal "Foo in nil", body
    end

    it 'should resolve views and layouts paths' do
      assert_equal Tennpipes.root('views')+'/users/index', TennpipesPristine.view_path('users/index')
      assert_equal Tennpipes.root('views')+'/layouts/app', TennpipesPristine.layout_path(:app)
    end

    describe "errors" do
      it 'should have not mapped errors on development' do
        mock_app { get('/'){ 'HI' } }
        get "/"
        assert @app.errors.empty?
      end

      it 'should have mapped errors on production' do
        mock_app { set :environment, :production; get('/'){ 'HI' } }
        get "/"
        assert_equal 1, @app.errors.size
      end

      it 'should overide errors' do
        mock_app do
          set :environment, :production
          get('/'){ raise }
          error(::Exception){ 'custom error' }
        end
        get "/"
        assert_equal 1, @app.errors.size
        assert_equal 'custom error', body
      end

      it 'should raise NameError even if Kernel.require is extended' do
        assert_raises NameError do
          ConstTest = Class.new(Tennpipes::Application)
          require 'active_support/dependencies'
          ConstTest::UninitializedConstant
        end
      end
    end

    describe "pre-compile routes" do
      it "should compile routes before first request if enabled the :precompile_routes option" do
        require File.expand_path(File.dirname(__FILE__) + '/fixtures/apps/precompiled_app')
        assert_instance_of Tennpipes::PathRouter::Compiler, PrecompiledApp::App.compiled_router.engine
        assert_instance_of Tennpipes::PathRouter::Compiler, PrecompiledApp::SubApp.compiled_router.engine
        assert_equal true, PrecompiledApp::App.compiled_router.engine.compiled?
        assert_equal true, PrecompiledApp::SubApp.compiled_router.engine.compiled?
        assert_equal 20, PrecompiledApp::App.compiled_router.engine.regexps.length
        assert_equal 20, PrecompiledApp::SubApp.compiled_router.engine.regexps.length
      end
    end

    describe 'global prerequisites' do
      after do
        Tennpipes::Application.prerequisites.clear
      end

      it 'should be inherited by children of Tennpipes::Application' do
        Tennpipes::Application.prerequisites << 'my_prerequisites'
        class InheritanceTest < Tennpipes::Application; end
        assert_includes InheritanceTest.prerequisites, 'my_prerequisites'
      end
    end
  end # application functionality
end
