require File.expand_path(File.dirname(__FILE__) + '/helper')
require 'logger'

describe "TennpipesLogger" do
  before do
    @save_config = Tennpipes::Logger::Config[:test].dup
    Tennpipes::Logger::Config[:test][:stream] = :null
    Tennpipes::Logger.setup!
  end

  after do
    Tennpipes::Logger::Config[:test] = @save_config
    Tennpipes::Logger.setup!
  end

  def setup_logger(options={})
    @log    = StringIO.new
    @logger = Tennpipes::Logger.new(options.merge(:stream => @log))
  end

  describe 'for logger functionality' do
    describe 'check stream config' do
      it 'should use stdout if stream is nil' do
        Tennpipes::Logger::Config[:test][:stream] = nil
        Tennpipes::Logger.setup!
        assert_equal $stdout, Tennpipes.logger.log
      end

      it 'should use StringIO as default for test' do
        assert_instance_of StringIO, Tennpipes.logger.log
      end

      it 'should use a custom stream' do
        my_stream = StringIO.new
        Tennpipes::Logger::Config[:test][:stream] = my_stream
        Tennpipes::Logger.setup!
        assert_equal my_stream, Tennpipes.logger.log
      end
    end

    it 'should log something' do
      setup_logger(:log_level => :error)
      @logger.error "You log this error?"
      assert_match(/You log this error?/, @log.string)
      @logger.debug "You don't log this error!"
      refute_match(/You don't log this error!/, @log.string)
      @logger << "Yep this can be logged"
      assert_match(/Yep this can be logged/, @log.string)
    end

    it 'should respond to #write for Rack::CommonLogger' do
      setup_logger(:log_level => :error)
      @logger.error "Error message"
      assert_match /Error message/, @log.string
      @logger << "logged anyways"
      assert_match /logged anyways/, @log.string
      @logger.write "log via alias"
      assert_match /log via alias/, @log.string
    end

    it 'should log an application' do
      mock_app do
        enable :logging
        get("/"){ "Foo" }
      end
      get "/"
      assert_equal "Foo", body
      assert_match /GET/, Tennpipes.logger.log.string
    end

    it 'should log an application\'s status code' do
      mock_app do
        enable :logging
        get("/"){ "Foo" }
      end
      get "/"
      assert_match /\e\[1;9m200\e\[0m OK/, Tennpipes.logger.log.string
    end

    describe "static asset logging" do
      it 'should not log static assets by default' do
        mock_app do
          enable :logging
          get("/images/something.png"){ env["sinatra.static_file"] = '/public/images/something.png'; "Foo" }
        end
        get "/images/something.png"
        assert_equal "Foo", body
        assert_match "", Tennpipes.logger.log.string
      end

      it 'should allow turning on static assets logging' do
        Tennpipes.logger.instance_eval{ @log_static = true }
        mock_app do
          enable :logging
          get("/images/something.png"){ env["sinatra.static_file"] = '/public/images/something.png'; "Foo" }
        end
        get "/images/something.png"
        assert_equal "Foo", body
        assert_match /GET/, Tennpipes.logger.log.string
        Tennpipes.logger.instance_eval{ @log_static = false }
      end
    end

    describe "health-check requests logging" do
      def access_to_mock_app
        mock_app do
          enable :logging
          get("/"){ "Foo" }
        end
        get "/"
      end

      it 'should output under debug level' do
        Tennpipes.logger.instance_eval{ @level = Tennpipes::Logger::Levels[:debug] }
        access_to_mock_app
        assert_match /\e\[0;36m  DEBUG\e\[0m/, Tennpipes.logger.log.string

        Tennpipes.logger.instance_eval{ @level = Tennpipes::Logger::Levels[:devel] }
        access_to_mock_app
        assert_match /\e\[0;36m  DEBUG\e\[0m/, Tennpipes.logger.log.string
      end

      it 'should not output over debug level' do
        Tennpipes.logger.instance_eval{ @level = Tennpipes::Logger::Levels[:info] }
        access_to_mock_app
        assert_equal '', Tennpipes.logger.log.string

        Tennpipes.logger.instance_eval{ @level = Tennpipes::Logger::Levels[:error] }
        access_to_mock_app
        assert_equal '', Tennpipes.logger.log.string
      end
    end
  end
end

describe "alternate logger" do
  class FancyLogger
    attr_accessor :level, :log
    def initialize(buf)
      self.log = buf
      self.level = 0
    end
    def add(level, text)
      self.log << text
    end
  end

  before do
    @save_logger = Tennpipes.logger
    @log = StringIO.new
    Tennpipes.logger = FancyLogger.new(@log)
  end

  after do
    Tennpipes.logger = @save_logger
  end

  it 'should annotate the logger to support additional Tennpipes fancyness' do
    Tennpipes.logger.debug("Debug message")
    assert_match(/Debug message/, @log.string)
    Tennpipes.logger.exception(Exception.new 'scary message')
    assert_match(/Exception - scary message/, @log.string)
  end

  it 'should colorize log output after colorize! is called' do
    Tennpipes.logger.colorize!

    mock_app do
      enable :logging
      get("/"){ "Foo" }
    end
    get "/"

    assert_match /\e\[1;9m200\e\[0m OK/, @log.string
  end
end

describe "alternate logger: stdlib logger" do
  before do
    @log = StringIO.new
    @save_logger = Tennpipes.logger
    Tennpipes.logger = Logger.new(@log)
  end

  after do
    Tennpipes.logger = @save_logger
  end

  it 'should annotate the logger to support additional Tennpipes fancyness' do
    Tennpipes.logger.debug("Debug message")
    assert_match(/Debug message/, @log.string)
  end

  it 'should colorize log output after colorize! is called' do
    Tennpipes.logger.colorize!

    mock_app do
      enable :logging
      get("/"){ "Foo" }
    end
    get "/"

    assert_match /\e\[1;9m200\e\[0m OK/, @log.string
  end
end

describe "options :colorize_logging" do
  def access_to_mock_app
    mock_app do
      enable :logging
      get("/"){ "Foo" }
    end
    get "/"
  end

  before do
    @save_config = Tennpipes::Logger::Config[:test].dup
  end

  after do
    Tennpipes::Logger::Config[:test] = @save_config
    Tennpipes::Logger.setup!
  end

  describe 'default' do
    before do
      Tennpipes::Logger::Config[:test][:colorize_logging] = true
      Tennpipes::Logger.setup!
    end

    it 'should use colorize logging' do
      Tennpipes::Logger.setup!

      access_to_mock_app
      assert_match /\e\[1;9m200\e\[0m OK/, Tennpipes.logger.log.string
    end
  end

  describe 'set value is false' do
    before do
      Tennpipes::Logger::Config[:test][:colorize_logging] = false
      Tennpipes::Logger.setup!
    end

    it 'should not use colorize logging' do
      access_to_mock_app
      assert_match /200 OK/, Tennpipes.logger.log.string
    end
  end
end

describe "options :source_location" do
  before do
    Tennpipes::Logger::Config[:test][:source_location] = true
    Tennpipes::Logger.setup!
  end

  def stub_root(base_path = File.expand_path("."), &block)
    callable = proc{ |*args| File.join(base_path, *args) }
    Tennpipes.stub(:root, callable, &block)
  end

  it 'should output source_location if :source_location is set to true' do
    stub_root { Tennpipes.logger.debug("hello world") }
    assert_match /\[test\/test_logger\.rb:264\] hello world/, Tennpipes.logger.log.string
  end

  it 'should output source_location if file path is relative' do
    stub_message = "test/test_logger.rb:269:in `test'"
    Tennpipes::Logger.logger.stub(:caller, [stub_message]){ stub_root { Tennpipes.logger.debug("hello relative path") }}
    assert_match /\[test\/test_logger\.rb:269\] hello relative path/, Tennpipes.logger.log.string
  end

  it 'should not output source_location if :source_location is set to false' do
    Tennpipes::Logger::Config[:test][:source_location] = false
    Tennpipes::Logger.setup!
    stub_root { Tennpipes.logger.debug("hello world") }
    assert_match /hello world/, Tennpipes.logger.log.string
    refute_match /\[.+?\] hello world/, Tennpipes.logger.log.string
  end

  it 'should not output source_location unless file path is not started with Tennpipes.root' do
    stub_root("/unknown/path/") { Tennpipes.logger.debug("hello boy") }
    assert_match /hello boy/, Tennpipes.logger.log.string
    refute_match /\[.+?\] hello boy/, Tennpipes.logger.log.string
  end

  it 'should not output source_location if source file path is started with Tennpipes.root + vendor' do
    base_path = File.expand_path(File.dirname(__FILE__) + '/fixtures/')
    stub_message = File.expand_path(File.dirname(__FILE__) + '/fixtures/vendor/logger.rb') + ":291:in `test'"
    Tennpipes::Logger.logger.stub(:caller, [stub_message]) { stub_root(base_path) { Tennpipes.logger.debug("hello vendor") } }
    assert_match /hello vendor/, Tennpipes.logger.log.string
    refute_match /\[.+?\] hello vendor/, Tennpipes.logger.log.string
  end
end
